@@ -1,29 +1,31 @@
-# Copyright (c) 2012 OpenStack, LLC.
-#
-# Licensed under the Apache License, Version 2.0 (the "License");
-# you may not use this file except in compliance with the License.
-# You may obtain a copy of the License at
-#
-#    http://www.apache.org/licenses/LICENSE-2.0
-#
-# Unless required by applicable law or agreed to in writing, software
-# distributed under the License is distributed on an "AS IS" BASIS,
-# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
-# implied.
-# See the License for the specific language governing permissions and
-# limitations under the License.
+# vim: tabstop=4 shiftwidth=4 softtabstop=4
 
-import logging
+# Copyright (c) 2012 OpenStack Foundation.
+# All Rights Reserved.
+#
+#    Licensed under the Apache License, Version 2.0 (the "License"); you may
+#    not use this file except in compliance with the License. You may obtain
+#    a copy of the License at
+#
+#         http://www.apache.org/licenses/LICENSE-2.0
+#
+#    Unless required by applicable law or agreed to in writing, software
+#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
+#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
+#    License for the specific language governing permissions and limitations
+#    under the License.
 
 import eventlet
-import uuid
 
 from quantum.common import topics
 
+from quantum.openstack.common import log as logging
+from quantum.openstack.common.notifier import api
+from quantum.openstack.common.notifier import rpc_notifier
 from quantum.openstack.common import rpc
 from quantum.openstack.common.rpc import proxy
-from quantum.openstack.common.notifier import api
-from quantum.openstack.common.notifier import rabbit_notifier
+from quantum.openstack.common import timeutils
+from quantum.openstack.common import uuidutils
 
 
 LOG = logging.getLogger(__name__)
@@ -48,6 +50,22 @@ def create_consumers(dispatcher, prefix, topic_details):
     return connection
 
 
+class PluginReportStateAPI(proxy.RpcProxy):
+    BASE_RPC_API_VERSION = '1.0'
+
+    def __init__(self, topic):
+        super(PluginReportStateAPI, self).__init__(
+            topic=topic, default_version=self.BASE_RPC_API_VERSION)
+
+    def report_state(self, context, agent_state):
+        return self.call(context,
+                         self.make_msg('report_state',
+                                       agent_state={'agent_state':
+                                                    agent_state},
+                                       time=timeutils.strtime()),
+                         topic=self.topic)
+
+
 class PluginApi(proxy.RpcProxy):
     '''Agent side of the rpc API.
 
@@ -84,34 +102,3 @@ class PluginApi(proxy.RpcProxy):
         return self.call(context,
                          self.make_msg('tunnel_sync', tunnel_ip=tunnel_ip),
                          topic=self.topic)
-
-
-class NotificationDispatcher(object):
-    def __init__(self):
-        # Set the Queue size to 1 so that messages stay on server rather than
-        # being buffered in the process.
-        self.queue = eventlet.queue.Queue(1)
-        self.connection = rpc.create_connection(new=True)
-        topic = '%s.%s' % (rabbit_notifier.CONF.notification_topics[0],
-                           api.CONF.default_notification_level.lower())
-        queue_name = 'notification_listener_%s' % str(uuid.uuid4())
-        self.connection.declare_topic_consumer(topic=topic,
-                                               queue_name=queue_name,
-                                               callback=self._add_to_queue)
-        self.connection.consume_in_thread()
-
-    def _add_to_queue(self, msg):
-        self.queue.put(msg)
-
-    def run_dispatch(self, handler):
-        while True:
-            msg = self.queue.get()
-            name = msg['event_type'].replace('.', '_')
-
-            try:
-                if hasattr(handler, name):
-                    getattr(handler, name)(msg['payload'])
-                else:
-                    LOG.debug('Unknown event_type: %s.' % msg['event_type'])
-            except Exception, e:
-                LOG.warn('Error processing message. Exception: %s' % e)
