@@ -1,219 +1,220 @@
-# Copyright (c) 2012 OpenStack, LLC.
+# vim: tabstop=4 shiftwidth=4 softtabstop=4
+
+# Copyright (c) 2012 OpenStack Foundation.
+# All Rights Reserved.
 #
-# Licensed under the Apache License, Version 2.0 (the 'License');
-# you may not use this file except in compliance with the License.
-# You may obtain a copy of the License at
+#    Licensed under the Apache License, Version 2.0 (the 'License'); you may
+#    not use this file except in compliance with the License. You may obtain
+#    a copy of the License at
 #
-#    http://www.apache.org/licenses/LICENSE-2.0
+#         http://www.apache.org/licenses/LICENSE-2.0
 #
-# Unless required by applicable law or agreed to in writing, software
-# distributed under the License is distributed on an 'AS IS' BASIS,
-# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
-# implied.
-# See the License for the specific language governing permissions and
-# limitations under the License.
+#    Unless required by applicable law or agreed to in writing, software
+#    distributed under the License is distributed on an 'AS IS' BASIS, WITHOUT
+#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
+#    License for the specific language governing permissions and limitations
+#    under the License.
+
+import netaddr
+import re
+
+from quantum.common import constants
+from quantum.common import exceptions as q_exc
+from quantum.openstack.common import log as logging
+from quantum.openstack.common import uuidutils
+
+
+LOG = logging.getLogger(__name__)
 
 ATTR_NOT_SPECIFIED = object()
 # Defining a constant to avoid repeating string literal in several modules
 SHARED = 'shared'
 
-import logging
-import netaddr
-import re
 
-from quantum.common import exceptions as q_exc
+def _verify_dict_keys(expected_keys, target_dict, strict=True):
+    """ Allows to verify keys in a dictionary.
+    :param expected_keys: A list of keys expected to be present.
+    :param target_dict: The dictionary which should be verified.
+    :param strict: Specifies whether additional keys are allowed to be present.
+    :return: True, if keys in the dictionary correspond to the specification.
+    """
+    if not isinstance(target_dict, dict):
+        msg = (_("Invalid input. '%(target_dict)s' must be a dictionary "
+                 "with keys: %(expected_keys)s") %
+               dict(target_dict=target_dict, expected_keys=expected_keys))
+        return msg
 
+    expected_keys = set(expected_keys)
+    provided_keys = set(target_dict.keys())
 
-LOG = logging.getLogger(__name__)
+    predicate = expected_keys.__eq__ if strict else expected_keys.issubset
 
-
-def is_attr_set(attribute):
-    return attribute not in (None, ATTR_NOT_SPECIFIED)
-
-
-def _validate_boolean(data, valid_values=None):
-    if data in [True, False]:
-        return
-    else:
-        msg = _("'%s' is not boolean") % data
-        LOG.debug("validate_boolean: %s", msg)
+    if not predicate(provided_keys):
+        msg = (_("Validation of dictionary's keys failed."
+                 "Expected keys: %(expected_keys)s "
+                 "Provided keys: %(provided_keys)s") % locals())
         return msg
 
 
+def is_attr_set(attribute):
+    return not (attribute is None or attribute is ATTR_NOT_SPECIFIED)
+
+
 def _validate_values(data, valid_values=None):
-    if data in valid_values:
-        return
-    else:
-        msg_dict = dict(data=data, values=valid_values)
-        msg = _("%(data)s is not in %(values)s") % msg_dict
-        LOG.debug("validate_values: %s", msg)
+    if data not in valid_values:
+        msg = (_("'%(data)s' is not in %(valid_values)s") %
+               dict(data=data, valid_values=valid_values))
+        LOG.debug(msg)
         return msg
 
 
 def _validate_string(data, max_len=None):
     if not isinstance(data, basestring):
         msg = _("'%s' is not a valid string") % data
-        LOG.debug("validate_string: %s", msg)
+        LOG.debug(msg)
+        return msg
+
+    if max_len is not None and len(data) > max_len:
+        msg = (_("'%(data)s' exceeds maximum length of %(max_len)s") %
+               dict(data=data, max_len=max_len))
+        LOG.debug(msg)
         return msg
-    if max_len is not None:
-        if len(data) > max_len:
-            msg = _("'%(data)s' exceeds maximum length of "
-                    "%(max_len)s.") % locals()
-            LOG.debug("validate_string: %s", msg)
-            return msg
 
 
 def _validate_range(data, valid_values=None):
     min_value = valid_values[0]
     max_value = valid_values[1]
-    if data >= min_value and data <= max_value:
-        return
-    else:
-        msg_dict = dict(data=data, min_value=min_value, max_value=max_value)
-        msg = _("%(data)s is not in range %(min_value)s through "
-                "%(max_value)s") % msg_dict
-        LOG.debug("validate_range: %s", msg)
+    if not min_value <= data <= max_value:
+        msg = _("'%(data)s' is not in range %(min_value)s through "
+                "%(max_value)s") % dict(data=data,
+                                        min_value=min_value,
+                                        max_value=max_value)
+        LOG.debug(msg)
         return msg
 
 
 def _validate_mac_address(data, valid_values=None):
     try:
         netaddr.EUI(data)
-        return
     except Exception:
         msg = _("'%s' is not a valid MAC address") % data
-        LOG.debug("validate_mac_address: %s", msg)
+        LOG.debug(msg)
         return msg
 
 
 def _validate_ip_address(data, valid_values=None):
     try:
         netaddr.IPAddress(data)
-        return
     except Exception:
         msg = _("'%s' is not a valid IP address") % data
-        LOG.debug("validate_ip_address: %s", msg)
+        LOG.debug(msg)
         return msg
 
 
 def _validate_ip_pools(data, valid_values=None):
     """Validate that start and end IP addresses are present
 
-    In addition to this the IP addresses will also be validated"""
+    In addition to this the IP addresses will also be validated
+
+    """
     if not isinstance(data, list):
-        msg = _("'%s' in not a valid IP pool") % data
-        LOG.debug("validate_ip_pools: %s", msg)
+        msg = _("Invalid data format for IP pool: '%s'") % data
+        LOG.debug(msg)
         return msg
 
-    expected_keys = set(['start', 'end'])
-    try:
-        for ip_pool in data:
-            if set(ip_pool.keys()) != expected_keys:
-                msg = _("Expected keys not found. Expected: %s "
-                        "Provided: %s") % (expected_keys, ip_pool.keys())
-                LOG.debug("validate_ip_pools: %s", msg)
+    expected_keys = ['start', 'end']
+    for ip_pool in data:
+        msg = _verify_dict_keys(expected_keys, ip_pool)
+        if msg:
+            LOG.debug(msg)
+            return msg
+        for k in expected_keys:
+            msg = _validate_ip_address(ip_pool[k])
+            if msg:
+                LOG.debug(msg)
                 return msg
-            for k in expected_keys:
-                msg = _validate_ip_address(ip_pool[k])
-                if msg:
-                    LOG.debug("validate_ip_pools: %s", msg)
-                    return msg
-    except KeyError, e:
-        args = {'key_name': e.message, 'ip_pool': ip_pool}
-        msg = _("Invalid input. Required key: '%(key_name)s' "
-                "missing from %(ip_pool)s.") % args
-        LOG.debug("validate_ip_pools: %s", msg)
-        return msg
-    except TypeError, e:
-        msg = _("Invalid input. Pool %s must be a dictionary.") % ip_pool
-        LOG.debug("validate_ip_pools: %s", msg)
-        return msg
-    except Exception:
-        msg = _("'%s' in not a valid IP pool") % data
-        LOG.debug("validate_ip_pools: %s", msg)
-        return msg
 
 
 def _validate_fixed_ips(data, valid_values=None):
     if not isinstance(data, list):
-        msg = _("'%s' in not a valid fixed IP") % data
-        LOG.debug("validate_fixed_ips: %s", msg)
+        msg = _("Invalid data format for fixed IP: '%s'") % data
+        LOG.debug(msg)
         return msg
+
     ips = []
-    try:
-        for fixed_ip in data:
-            if 'ip_address' in fixed_ip:
-                msg = _validate_ip_address(fixed_ip['ip_address'])
-                if msg:
-                    LOG.debug("validate_fixed_ips: %s", msg)
-                    return msg
-            if 'subnet_id' in fixed_ip:
-                msg = _validate_regex(fixed_ip['subnet_id'], UUID_PATTERN)
-                if msg:
-                    LOG.debug("validate_fixed_ips: %s", msg)
-                    return msg
+    for fixed_ip in data:
+        if not isinstance(fixed_ip, dict):
+            msg = _("Invalid data format for fixed IP: '%s'") % fixed_ip
+            LOG.debug(msg)
+            return msg
+        if 'ip_address' in fixed_ip:
             # Ensure that duplicate entries are not set - just checking IP
             # suffices. Duplicate subnet_id's are legitimate.
-            if 'ip_address' in fixed_ip:
-                if fixed_ip['ip_address'] in ips:
-                    msg = _("Duplicate entry %s") % fixed_ip
-                    LOG.debug("validate_fixed_ips: %s", msg)
-                    return msg
-                ips.append(fixed_ip['ip_address'])
-    except Exception:
-        msg = _("'%s' in not a valid fixed IP") % data
-        LOG.debug("validate_fixed_ips: %s", msg)
-        return msg
+            fixed_ip_address = fixed_ip['ip_address']
+            if fixed_ip_address in ips:
+                msg = _("Duplicate IP address '%s'") % fixed_ip_address
+            else:
+                msg = _validate_ip_address(fixed_ip_address)
+            if msg:
+                LOG.debug(msg)
+                return msg
+            ips.append(fixed_ip_address)
+        if 'subnet_id' in fixed_ip:
+            msg = _validate_uuid(fixed_ip['subnet_id'])
+            if msg:
+                LOG.debug(msg)
+                return msg
 
 
 def _validate_nameservers(data, valid_values=None):
     if not hasattr(data, '__iter__'):
-        msg = _("'%s' in not a valid nameserver") % data
-        LOG.debug("validate_nameservers: %s", msg)
+        msg = _("Invalid data format for nameserver: '%s'") % data
+        LOG.debug(msg)
         return msg
-    ips = set()
+
+    ips = []
     for ip in data:
         msg = _validate_ip_address(ip)
         if msg:
             # This may be a hostname
             msg = _validate_regex(ip, HOSTNAME_PATTERN)
             if msg:
-                msg = _("'%s' in not a valid nameserver") % ip
-                LOG.debug("validate_nameservers: %s", msg)
+                msg = _("'%s' is not a valid nameserver") % ip
+                LOG.debug(msg)
                 return msg
         if ip in ips:
-            msg = _("Duplicate nameserver %s") % ip
-            LOG.debug("validate_nameservers: %s", msg)
+            msg = _("Duplicate nameserver '%s'") % ip
+            LOG.debug(msg)
             return msg
-        ips.add(ip)
+        ips.append(ip)
 
 
 def _validate_hostroutes(data, valid_values=None):
     if not isinstance(data, list):
-        msg = _("'%s' in not a valid hostroute") % data
-        LOG.debug("validate_hostroutes: %s", msg)
+        msg = _("Invalid data format for hostroute: '%s'") % data
+        LOG.debug(msg)
         return msg
+
+    expected_keys = ['destination', 'nexthop']
     hostroutes = []
-    try:
-        for hostroute in data:
-            msg = _validate_subnet(hostroute['destination'])
-            if msg:
-                LOG.debug("validate_hostroutes: %s", msg)
-                return msg
-            msg = _validate_ip_address(hostroute['nexthop'])
-            if msg:
-                LOG.debug("validate_hostroutes: %s", msg)
-                return msg
-            if hostroute in hostroutes:
-                msg = _("Duplicate hostroute %s") % hostroute
-                LOG.debug("validate_hostroutes: %s", msg)
-                if msg:
-                    return msg
-            hostroutes.append(hostroute)
-    except:
-        msg = _("'%s' in not a valid hostroute") % data
-        LOG.debug("validate_hostroutes: %s", msg)
-        return msg
+    for hostroute in data:
+        msg = _verify_dict_keys(expected_keys, hostroute)
+        if msg:
+            LOG.debug(msg)
+            return msg
+        msg = _validate_subnet(hostroute['destination'])
+        if msg:
+            LOG.debug(msg)
+            return msg
+        msg = _validate_ip_address(hostroute['nexthop'])
+        if msg:
+            LOG.debug(msg)
+            return msg
+        if hostroute in hostroutes:
+            msg = _("Duplicate hostroute '%s'") % hostroute
+            LOG.debug(msg)
+            return msg
+        hostroutes.append(hostroute)
 
 
 def _validate_ip_address_or_none(data, valid_values=None):
@@ -231,7 +232,7 @@ def _validate_subnet(data, valid_values=None):
         pass
 
     msg = _("'%s' is not a valid IP subnet") % data
-    LOG.debug("validate_subnet: %s", msg)
+    LOG.debug(msg)
     return msg
 
 
@@ -242,26 +243,121 @@ def _validate_regex(data, valid_values=None):
     except TypeError:
         pass
 
-    msg = _("'%s' is not valid input") % data
-    LOG.debug("validate_regex: %s", msg)
+    msg = _("'%s' is not a valid input") % data
+    LOG.debug(msg)
     return msg
 
 
-def convert_to_boolean(data):
+def _validate_uuid(data, valid_values=None):
+    if not uuidutils.is_uuid_like(data):
+        msg = _("'%s' is not a valid UUID") % data
+        LOG.debug(msg)
+        return msg
+
+
+def _validate_uuid_or_none(data, valid_values=None):
+    if data is not None:
+        return _validate_uuid(data)
+
+
+def _validate_uuid_list(data, valid_values=None):
+    if not isinstance(data, list):
+        msg = _("'%s' is not a list") % data
+        LOG.debug(msg)
+        return msg
+
+    for item in data:
+        msg = _validate_uuid(item)
+        if msg:
+            LOG.debug(msg)
+            return msg
+
+    if len(set(data)) != len(data):
+        msg = _("Duplicate items in the list: '%s'") % ', '.join(data)
+        LOG.debug(msg)
+        return msg
+
+
+def _validate_dict(data, key_specs=None):
+    if not isinstance(data, dict):
+        msg = _("'%s' is not a dictionary") % data
+        LOG.debug(msg)
+        return msg
+
+    # Do not perform any further validation, if no constraints are supplied
+    if not key_specs:
+        return
+
+    # Check whether all required keys are present
+    required_keys = [key for key, spec in key_specs.iteritems()
+                     if spec.get('required')]
+
+    if required_keys:
+        msg = _verify_dict_keys(required_keys, data, False)
+        if msg:
+            LOG.debug(msg)
+            return msg
+
+    # Perform validation of all values according to the specifications.
+    for key, key_validator in [(k, v) for k, v in key_specs.iteritems()
+                               if k in data]:
+
+        for val_name in [n for n in key_validator.iterkeys()
+                         if n.startswith('type:')]:
+            # Check whether specified validator exists.
+            if val_name not in validators:
+                msg = _("Validator '%s' does not exist.") % val_name
+                LOG.debug(msg)
+                return msg
+
+            val_func = validators[val_name]
+            val_params = key_validator[val_name]
+
+            msg = val_func(data.get(key), val_params)
+            if msg:
+                LOG.debug(msg)
+                return msg
+
+
+def _validate_dict_or_none(data, key_specs=None):
+    if data is not None:
+        return _validate_dict(data, key_specs)
+
+
+def _validate_dict_or_empty(data, key_specs=None):
+    if data != {}:
+        return _validate_dict(data, key_specs)
+
+
+def _validate_non_negative(data, valid_values=None):
     try:
-        i = int(data)
-        if i in [True, False]:
-            # Ensure that the value is True or False
-            if i:
-                return True
-            else:
-                return False
+        data = int(data)
     except (ValueError, TypeError):
-        if (data == "True" or data == "true"):
+        msg = _("'%s' is not an integer") % data
+        LOG.debug(msg)
+        return msg
+
+    if data < 0:
+        msg = _("'%s' should be non-negative") % data
+        LOG.debug(msg)
+        return msg
+
+
+def convert_to_boolean(data):
+    if isinstance(data, basestring):
+        val = data.lower()
+        if val == "true" or val == "1":
             return True
-        if (data == "False" or data == "false"):
+        if val == "false" or val == "0":
             return False
-    msg = _("'%s' is not boolean") % data
+    elif isinstance(data, bool):
+        return data
+    elif isinstance(data, int):
+        if data == 0:
+            return False
+        elif data == 1:
+            return True
+    msg = _("'%s' cannot be converted to boolean") % data
     raise q_exc.InvalidInput(error_message=msg)
 
 
@@ -303,6 +399,24 @@ def convert_kvp_list_to_dict(kvp_list):
         kvp_map[key].add(value)
     return dict((x, list(y)) for x, y in kvp_map.iteritems())
 
+
+def convert_none_to_empty_list(value):
+    return [] if value is None else value
+
+
+def convert_none_to_empty_dict(value):
+    return {} if value is None else value
+
+
+def convert_to_list(data):
+    if data is None:
+        return []
+    elif hasattr(data, '__iter__'):
+        return list(data)
+    else:
+        return [data]
+
+
 HOSTNAME_PATTERN = ("(?=^.{1,254}$)(^(?:(?!\d+\.|-)[a-zA-Z0-9_\-]"
                     "{1,63}(?<!-)\.?)+(?:[a-zA-Z]{2,})$)")
 
@@ -315,19 +429,25 @@ UUID_PATTERN = '-'.join([HEX_ELEM + '{8}', HEX_ELEM + '{4}',
 MAC_PATTERN = "^%s[aceACE02468](:%s{2}){5}$" % (HEX_ELEM, HEX_ELEM)
 
 # Dictionary that maintains a list of validation functions
-validators = {'type:boolean': _validate_boolean,
-              'type:values': _validate_values,
-              'type:string': _validate_string,
-              'type:range': _validate_range,
-              'type:mac_address': _validate_mac_address,
+validators = {'type:dict': _validate_dict,
+              'type:dict_or_none': _validate_dict_or_none,
+              'type:dict_or_empty': _validate_dict_or_empty,
               'type:fixed_ips': _validate_fixed_ips,
+              'type:hostroutes': _validate_hostroutes,
               'type:ip_address': _validate_ip_address,
               'type:ip_address_or_none': _validate_ip_address_or_none,
-              'type:subnet': _validate_subnet,
-              'type:regex': _validate_regex,
               'type:ip_pools': _validate_ip_pools,
-              'type:hostroutes': _validate_hostroutes,
-              'type:nameservers': _validate_nameservers}
+              'type:mac_address': _validate_mac_address,
+              'type:nameservers': _validate_nameservers,
+              'type:non_negative': _validate_non_negative,
+              'type:range': _validate_range,
+              'type:regex': _validate_regex,
+              'type:string': _validate_string,
+              'type:subnet': _validate_subnet,
+              'type:uuid': _validate_uuid,
+              'type:uuid_or_none': _validate_uuid_or_none,
+              'type:uuid_list': _validate_uuid_list,
+              'type:values': _validate_values}
 
 # Note: a default of ATTR_NOT_SPECIFIED indicates that an
 # attribute is not required, but will be generated by the plugin
@@ -355,8 +475,9 @@ validators = {'type:boolean': _validate_boolean,
 RESOURCE_ATTRIBUTE_MAP = {
     'networks': {
         'id': {'allow_post': False, 'allow_put': False,
-               'validate': {'type:regex': UUID_PATTERN},
-               'is_visible': True},
+               'validate': {'type:uuid': None},
+               'is_visible': True,
+               'primary_key': True},
         'name': {'allow_post': True, 'allow_put': True,
                  'validate': {'type:string': None},
                  'default': '', 'is_visible': True},
@@ -366,7 +487,6 @@ RESOURCE_ATTRIBUTE_MAP = {
         'admin_state_up': {'allow_post': True, 'allow_put': True,
                            'default': True,
                            'convert_to': convert_to_boolean,
-                           'validate': {'type:boolean': None},
                            'is_visible': True},
         'status': {'allow_post': False, 'allow_put': False,
                    'is_visible': True},
@@ -378,26 +498,25 @@ RESOURCE_ATTRIBUTE_MAP = {
                  'allow_put': True,
                  'default': False,
                  'convert_to': convert_to_boolean,
-                 'validate': {'type:boolean': None},
                  'is_visible': True,
                  'required_by_policy': True,
                  'enforce_policy': True},
     },
     'ports': {
         'id': {'allow_post': False, 'allow_put': False,
-               'validate': {'type:regex': UUID_PATTERN},
-               'is_visible': True},
+               'validate': {'type:uuid': None},
+               'is_visible': True,
+               'primary_key': True},
         'name': {'allow_post': True, 'allow_put': True, 'default': '',
                  'validate': {'type:string': None},
                  'is_visible': True},
         'network_id': {'allow_post': True, 'allow_put': False,
                        'required_by_policy': True,
-                       'validate': {'type:regex': UUID_PATTERN},
+                       'validate': {'type:uuid': None},
                        'is_visible': True},
         'admin_state_up': {'allow_post': True, 'allow_put': True,
                            'default': True,
                            'convert_to': convert_to_boolean,
-                           'validate': {'type:boolean': None},
                            'is_visible': True},
         'mac_address': {'allow_post': True, 'allow_put': False,
                         'default': ATTR_NOT_SPECIFIED,
@@ -427,8 +546,9 @@ RESOURCE_ATTRIBUTE_MAP = {
     },
     'subnets': {
         'id': {'allow_post': False, 'allow_put': False,
-               'validate': {'type:regex': UUID_PATTERN},
-               'is_visible': True},
+               'validate': {'type:uuid': None},
+               'is_visible': True,
+               'primary_key': True},
         'name': {'allow_post': True, 'allow_put': True, 'default': '',
                  'validate': {'type:string': None},
                  'is_visible': True},
@@ -438,7 +558,7 @@ RESOURCE_ATTRIBUTE_MAP = {
                        'is_visible': True},
         'network_id': {'allow_post': True, 'allow_put': False,
                        'required_by_policy': True,
-                       'validate': {'type:regex': UUID_PATTERN},
+                       'validate': {'type:uuid': None},
                        'is_visible': True},
         'cidr': {'allow_post': True, 'allow_put': False,
                  'validate': {'type:subnet': None},
@@ -453,10 +573,12 @@ RESOURCE_ATTRIBUTE_MAP = {
                              'validate': {'type:ip_pools': None},
                              'is_visible': True},
         'dns_nameservers': {'allow_post': True, 'allow_put': True,
+                            'convert_to': convert_none_to_empty_list,
                             'default': ATTR_NOT_SPECIFIED,
                             'validate': {'type:nameservers': None},
                             'is_visible': True},
         'host_routes': {'allow_post': True, 'allow_put': True,
+                        'convert_to': convert_none_to_empty_list,
                         'default': ATTR_NOT_SPECIFIED,
                         'validate': {'type:hostroutes': None},
                         'is_visible': True},
@@ -467,13 +589,11 @@ RESOURCE_ATTRIBUTE_MAP = {
         'enable_dhcp': {'allow_post': True, 'allow_put': True,
                         'default': True,
                         'convert_to': convert_to_boolean,
-                        'validate': {'type:boolean': None},
                         'is_visible': True},
         SHARED: {'allow_post': False,
                  'allow_put': False,
                  'default': False,
                  'convert_to': convert_to_boolean,
-                 'validate': {'type:boolean': None},
                  'is_visible': False,
                  'required_by_policy': True,
                  'enforce_policy': True},
@@ -487,3 +607,19 @@ RESOURCE_HIERARCHY_MAP = {
     'ports': {'parent': 'networks', 'identified_by': 'network_id'},
     'subnets': {'parent': 'networks', 'identified_by': 'network_id'}
 }
+
+PLURALS = {'networks': 'network',
+           'ports': 'port',
+           'subnets': 'subnet',
+           'dns_nameservers': 'dns_nameserver',
+           'host_routes': 'host_route',
+           'allocation_pools': 'allocation_pool',
+           'fixed_ips': 'fixed_ip',
+           'extensions': 'extension'}
+EXT_NSES = {}
+
+
+def get_attr_metadata():
+    return {'plurals': PLURALS,
+            'xmlns': constants.XML_NS_V20,
+            constants.EXT_NS: EXT_NSES}
