@@ -1,6 +1,5 @@
-"""
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
-#
+
 # Copyright 2012 Nicira Networks, Inc.  All rights reserved.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -17,41 +16,33 @@
 #
 # @author: Dan Wendlandt, Nicira, Inc
 #
-"""
-
-import logging
 
 import netaddr
 import sqlalchemy as sa
 from sqlalchemy import orm
 from sqlalchemy.orm import exc
 from sqlalchemy.sql import expression as expr
-import webob.exc as w_exc
 
+from quantum.api.rpc.agentnotifiers import l3_rpc_agent_api
 from quantum.api.v2 import attributes
+from quantum.common import constants as l3_constants
 from quantum.common import exceptions as q_exc
-from quantum.common import utils
 from quantum.db import db_base_plugin_v2
 from quantum.db import model_base
 from quantum.db import models_v2
 from quantum.extensions import l3
-from quantum.openstack.common import cfg
+from quantum.openstack.common import log as logging
+from quantum.openstack.common.notifier import api as notifier_api
+from quantum.openstack.common import uuidutils
 from quantum import policy
 
 
 LOG = logging.getLogger(__name__)
 
-l3_opts = [
-    cfg.StrOpt('metadata_ip_address', default='127.0.0.1'),
-    cfg.IntOpt('metadata_port', default=8775)
-]
 
-# Register the configuration options
-cfg.CONF.register_opts(l3_opts)
-
-DEVICE_OWNER_ROUTER_INTF = "network:router_interface"
-DEVICE_OWNER_ROUTER_GW = "network:router_gateway"
-DEVICE_OWNER_FLOATINGIP = "network:floatingip"
+DEVICE_OWNER_ROUTER_INTF = l3_constants.DEVICE_OWNER_ROUTER_INTF
+DEVICE_OWNER_ROUTER_GW = l3_constants.DEVICE_OWNER_ROUTER_GW
+DEVICE_OWNER_FLOATINGIP = l3_constants.DEVICE_OWNER_FLOATINGIP
 
 
 class Router(model_base.BASEV2, models_v2.HasId, models_v2.HasTenant):
@@ -101,6 +92,14 @@ class L3_NAT_db_mixin(l3.RouterPluginBase):
                                   *conditions)
         return conditions
 
+    def _network_result_filter_hook(self, query, filters):
+        vals = filters and filters.get('router:external', [])
+        if not vals:
+            return query
+        if vals[0]:
+            return query.filter((ExternalNetwork.network_id != expr.null()))
+        return query.filter((ExternalNetwork.network_id == expr.null()))
+
     # TODO(salvatore-orlando): Perform this operation without explicitly
     # referring to db_base_plugin_v2, as plugins that do not extend from it
     # might exist in the future
@@ -108,7 +107,8 @@ class L3_NAT_db_mixin(l3.RouterPluginBase):
         models_v2.Network,
         "external_net",
         _network_model_hook,
-        _network_filter_hook)
+        _network_filter_hook,
+        _network_result_filter_hook)
 
     def _get_router(self, context, id):
         try:
@@ -116,7 +116,7 @@ class L3_NAT_db_mixin(l3.RouterPluginBase):
         except exc.NoResultFound:
             raise l3.RouterNotFound(router_id=id)
         except exc.MultipleResultsFound:
-            LOG.error('Multiple routers match for %s' % id)
+            LOG.error(_('Multiple routers match for %s'), id)
             raise l3.RouterNotFound(router_id=id)
         return router
 
@@ -143,7 +143,7 @@ class L3_NAT_db_mixin(l3.RouterPluginBase):
         with context.session.begin(subtransactions=True):
             # pre-generate id so it will be available when
             # configuring external gw port
-            router_db = Router(id=utils.str_uuid(),
+            router_db = Router(id=uuidutils.generate_uuid(),
                                tenant_id=tenant_id,
                                name=r['name'],
                                admin_state_up=r['admin_state_up'],
@@ -167,6 +167,9 @@ class L3_NAT_db_mixin(l3.RouterPluginBase):
             # Ensure we actually have something to update
             if r.keys():
                 router_db.update(r)
+        routers = self.get_sync_data(context.elevated(),
+                                     [router_db['id']])
+        l3_rpc_agent_api.L3AgentNotify.routers_updated(context, routers)
         return self._make_router_dict(router_db)
 
     def _update_router_gw_info(self, context, router_id, info):
@@ -180,7 +183,8 @@ class L3_NAT_db_mixin(l3.RouterPluginBase):
         if network_id:
             self._get_network(context, network_id)
             if not self._network_is_external(context, network_id):
-                msg = "Network %s is not a valid external network" % network_id
+                msg = _("Network %s is not a valid external "
+                        "network") % network_id
                 raise q_exc.BadRequest(resource='router', msg=msg)
 
         # figure out if we need to delete existing port
@@ -220,7 +224,7 @@ class L3_NAT_db_mixin(l3.RouterPluginBase):
             if not len(gw_port['fixed_ips']):
                 self.delete_port(context.elevated(), gw_port['id'],
                                  l3_port_check=False)
-                msg = ('No IPs available for external network %s' %
+                msg = (_('No IPs available for external network %s') %
                        network_id)
                 raise q_exc.BadRequest(resource='router', msg=msg)
 
@@ -254,15 +258,23 @@ class L3_NAT_db_mixin(l3.RouterPluginBase):
                 self._delete_port(context.elevated(), ports[0]['id'])
 
             context.session.delete(router)
+        l3_rpc_agent_api.L3AgentNotify.router_deleted(context, id)
 
     def get_router(self, context, id, fields=None):
         router = self._get_router(context, id)
         return self._make_router_dict(router, fields)
 
-    def get_routers(self, context, filters=None, fields=None):
+    def get_routers(self, context, filters=None, fields=None,
+                    sorts=None, limit=None, marker=None,
+                    page_reverse=False):
+        marker_obj = self._get_marker_obj(context, 'router', limit, marker)
         return self._get_collection(context, Router,
                                     self._make_router_dict,
-                                    filters=filters, fields=fields)
+                                    filters=filters, fields=fields,
+                                    sorts=sorts,
+                                    limit=limit,
+                                    marker_obj=marker_obj,
+                                    page_reverse=page_reverse)
 
     def get_routers_count(self, context, filters=None):
         return self._get_collection_count(context, Router,
@@ -280,7 +292,7 @@ class L3_NAT_db_mixin(l3.RouterPluginBase):
             for p in rports:
                 for ip in p['fixed_ips']:
                     if ip['subnet_id'] == subnet_id:
-                        msg = ("Router already has a port on subnet %s"
+                        msg = (_("Router already has a port on subnet %s")
                                % subnet_id)
                         raise q_exc.BadRequest(resource='router', msg=msg)
                     sub_id = ip['subnet_id']
@@ -292,7 +304,7 @@ class L3_NAT_db_mixin(l3.RouterPluginBase):
                     if match1 or match2:
                         msg = (_("Cidr %(subnet_cidr)s of subnet "
                                  "%(subnet_id)s overlaps with cidr %(cidr)s "
-                                 " of subnet %(sub_id)") % locals())
+                                 "of subnet %(sub_id)s") % locals())
                         raise q_exc.BadRequest(resource='router', msg=msg)
         except exc.NoResultFound:
             pass
@@ -301,7 +313,7 @@ class L3_NAT_db_mixin(l3.RouterPluginBase):
         # make sure router exists
         router = self._get_router(context, router_id)
         if not interface_info:
-            msg = "Either subnet_id or port_id must be specified"
+            msg = _("Either subnet_id or port_id must be specified")
             raise q_exc.BadRequest(resource='router', msg=msg)
 
         try:
@@ -313,7 +325,7 @@ class L3_NAT_db_mixin(l3.RouterPluginBase):
 
         if 'port_id' in interface_info:
             if 'subnet_id' in interface_info:
-                msg = "cannot specify both subnet-id and port-id"
+                msg = _("Cannot specify both subnet-id and port-id")
                 raise q_exc.BadRequest(resource='router', msg=msg)
 
             port = self._get_port(context, interface_info['port_id'])
@@ -323,22 +335,22 @@ class L3_NAT_db_mixin(l3.RouterPluginBase):
                                       device_id=port['device_id'])
             fixed_ips = [ip for ip in port['fixed_ips']]
             if len(fixed_ips) != 1:
-                msg = 'Router port must have exactly one fixed IP'
+                msg = _('Router port must have exactly one fixed IP')
                 raise q_exc.BadRequest(resource='router', msg=msg)
-            subnet = self._get_subnet(context, fixed_ips[0]['subnet_id'])
+            subnet_id = fixed_ips[0]['subnet_id']
+            subnet = self._get_subnet(context, subnet_id)
             self._check_for_dup_router_subnet(context, router_id,
                                               port['network_id'],
                                               subnet['id'],
                                               subnet['cidr'])
-            with context.session.begin(subtransactions=True):
-                port.update({'device_id': router_id,
-                             'device_owner': DEVICE_OWNER_ROUTER_INTF})
+            port.update({'device_id': router_id,
+                         'device_owner': DEVICE_OWNER_ROUTER_INTF})
         elif 'subnet_id' in interface_info:
             subnet_id = interface_info['subnet_id']
             subnet = self._get_subnet(context, subnet_id)
             # Ensure the subnet has a gateway
             if not subnet['gateway_ip']:
-                msg = 'Subnet for router interface must have a gateway IP'
+                msg = _('Subnet for router interface must have a gateway IP')
                 raise q_exc.BadRequest(resource='router', msg=msg)
             self._check_for_dup_router_subnet(context, router_id,
                                               subnet['network_id'],
@@ -356,8 +368,20 @@ class L3_NAT_db_mixin(l3.RouterPluginBase):
                  'device_id': router_id,
                  'device_owner': DEVICE_OWNER_ROUTER_INTF,
                  'name': ''}})
-        return {'port_id': port['id'],
+
+        routers = self.get_sync_data(context.elevated(), [router_id])
+        l3_rpc_agent_api.L3AgentNotify.routers_updated(
+            context, routers, 'add_router_interface',
+            {'network_id': port['network_id'],
+             'subnet_id': subnet_id})
+        info = {'port_id': port['id'],
                 'subnet_id': port['fixed_ips'][0]['subnet_id']}
+        notifier_api.notify(context,
+                            notifier_api.publisher_id('network'),
+                            'router.interface.create',
+                            notifier_api.CONF.default_notification_level,
+                            {'router.interface': info})
+        return info
 
     def _confirm_router_interface_not_in_use(self, context, router_id,
                                              subnet_id):
@@ -380,29 +404,25 @@ class L3_NAT_db_mixin(l3.RouterPluginBase):
             raise l3.RouterNotFound(router_id=router_id)
 
         if not interface_info:
-            msg = "Either subnet_id or port_id must be specified"
+            msg = _("Either subnet_id or port_id must be specified")
             raise q_exc.BadRequest(resource='router', msg=msg)
         if 'port_id' in interface_info:
             port_id = interface_info['port_id']
             port_db = self._get_port(context, port_id)
             if not (port_db['device_owner'] == DEVICE_OWNER_ROUTER_INTF and
                     port_db['device_id'] == router_id):
-                raise w_exc.HTTPNotFound("Router %(router_id)s does not have "
-                                         " an interface with id %(port_id)s"
-                                         % locals())
+                raise l3.RouterInterfaceNotFound(router_id=router_id,
+                                                 port_id=port_id)
             if 'subnet_id' in interface_info:
                 port_subnet_id = port_db['fixed_ips'][0]['subnet_id']
                 if port_subnet_id != interface_info['subnet_id']:
-                    raise w_exc.HTTPConflict("subnet_id %s on port does not "
-                                             "match requested one (%s)"
-                                             % (port_subnet_id,
-                                                interface_info['subnet_id']))
-            if port_db['device_id'] != router_id:
-                raise w_exc.HTTPConflict("port_id %s not used by router" %
-                                         port_db['id'])
+                    raise q_exc.SubnetMismatchForPort(
+                        port_id=port_id,
+                        subnet_id=interface_info['subnet_id'])
+            subnet_id = port_db['fixed_ips'][0]['subnet_id']
             self._confirm_router_interface_not_in_use(
-                context, router_id,
-                port_db['fixed_ips'][0]['subnet_id'])
+                context, router_id, subnet_id)
+            _network_id = port_db['network_id']
             self.delete_port(context, port_db['id'], l3_port_check=False)
         elif 'subnet_id' in interface_info:
             subnet_id = interface_info['subnet_id']
@@ -421,6 +441,8 @@ class L3_NAT_db_mixin(l3.RouterPluginBase):
 
                 for p in ports:
                     if p['fixed_ips'][0]['subnet_id'] == subnet_id:
+                        port_id = p['id']
+                        _network_id = p['network_id']
                         self.delete_port(context, p['id'], l3_port_check=False)
                         found = True
                         break
@@ -428,9 +450,20 @@ class L3_NAT_db_mixin(l3.RouterPluginBase):
                 pass
 
             if not found:
-                raise w_exc.HTTPNotFound("Router %(router_id)s has no "
-                                         "interface on subnet %(subnet_id)s"
-                                         % locals())
+                raise l3.RouterInterfaceNotFoundForSubnet(router_id=router_id,
+                                                          subnet_id=subnet_id)
+        routers = self.get_sync_data(context.elevated(), [router_id])
+        l3_rpc_agent_api.L3AgentNotify.routers_updated(
+            context, routers, 'remove_router_interface',
+            {'network_id': _network_id,
+             'subnet_id': subnet_id})
+        notifier_api.notify(context,
+                            notifier_api.publisher_id('network'),
+                            'router.interface.delete',
+                            notifier_api.CONF.default_notification_level,
+                            {'router.interface':
+                             {'port_id': port_id,
+                              'subnet_id': subnet_id}})
 
     def _get_floatingip(self, context, id):
         try:
@@ -438,7 +471,7 @@ class L3_NAT_db_mixin(l3.RouterPluginBase):
         except exc.NoResultFound:
             raise l3.FloatingIPNotFound(floatingip_id=id)
         except exc.MultipleResultsFound:
-            LOG.error('Multiple floating ips match for %s' % id)
+            LOG.error(_('Multiple floating ips match for %s'), id)
             raise l3.FloatingIPNotFound(floatingip_id=id)
         return floatingip
 
@@ -457,8 +490,8 @@ class L3_NAT_db_mixin(l3.RouterPluginBase):
                                    external_network_id):
         subnet_db = self._get_subnet(context, internal_subnet_id)
         if not subnet_db['gateway_ip']:
-            msg = ('Cannot add floating IP to port on subnet %s '
-                   'which has no gateway_ip' % internal_subnet_id)
+            msg = (_('Cannot add floating IP to port on subnet %s '
+                     'which has no gateway_ip') % internal_subnet_id)
             raise q_exc.BadRequest(resource='floatingip', msg=msg)
 
         # find router interface ports on this network
@@ -511,18 +544,19 @@ class L3_NAT_db_mixin(l3.RouterPluginBase):
                 if ip['ip_address'] == internal_ip_address:
                     internal_subnet_id = ip['subnet_id']
             if not internal_subnet_id:
-                msg = ('Port %s does not have fixed ip %s' %
-                       (internal_port['id'], internal_ip_address))
+                msg = (_('Port %(id)s does not have fixed ip %(address)s') %
+                       {'id': internal_port['id'],
+                        'address': internal_ip_address})
                 raise q_exc.BadRequest(resource='floatingip', msg=msg)
         else:
             ips = [ip['ip_address'] for ip in internal_port['fixed_ips']]
             if len(ips) == 0:
-                msg = ('Cannot add floating IP to port %s that has'
-                       'no fixed IP addresses' % internal_port['id'])
+                msg = (_('Cannot add floating IP to port %s that has'
+                         'no fixed IP addresses') % internal_port['id'])
                 raise q_exc.BadRequest(resource='floatingip', msg=msg)
             if len(ips) > 1:
-                msg = ('Port %s has multiple fixed IPs.  Must provide'
-                       ' a specific IP when assigning a floating IP' %
+                msg = (_('Port %s has multiple fixed IPs.  Must provide'
+                         ' a specific IP when assigning a floating IP') %
                        internal_port['id'])
                 raise q_exc.BadRequest(resource='floatingip', msg=msg)
             internal_ip_address = internal_port['fixed_ips'][0]['ip_address']
@@ -551,7 +585,7 @@ class L3_NAT_db_mixin(l3.RouterPluginBase):
         port_id = internal_ip_address = router_id = None
         if (('fixed_ip_address' in fip and fip['fixed_ip_address']) and
             not ('port_id' in fip and fip['port_id'])):
-            msg = "fixed_ip_address cannot be specified without a port_id"
+            msg = _("fixed_ip_address cannot be specified without a port_id")
             raise q_exc.BadRequest(resource='floatingip', msg=msg)
         if 'port_id' in fip and fip['port_id']:
             port_id, internal_ip_address, router_id = self.get_assoc_data(
@@ -579,18 +613,18 @@ class L3_NAT_db_mixin(l3.RouterPluginBase):
     def create_floatingip(self, context, floatingip):
         fip = floatingip['floatingip']
         tenant_id = self._get_tenant_id_for_create(context, fip)
-        fip_id = utils.str_uuid()
+        fip_id = uuidutils.generate_uuid()
 
         f_net_id = fip['floating_network_id']
         if not self._network_is_external(context, f_net_id):
-            msg = "Network %s is not a valid external network" % f_net_id
+            msg = _("Network %s is not a valid external network") % f_net_id
             raise q_exc.BadRequest(resource='floatingip', msg=msg)
 
-        # This external port is never exposed to the tenant.
-        # it is used purely for internal system and admin use when
-        # managing floating IPs.
         try:
             with context.session.begin(subtransactions=True):
+                # This external port is never exposed to the tenant.
+                # it is used purely for internal system and admin use when
+                # managing floating IPs.
                 external_port = self.create_port(context.elevated(), {
                     'port':
                     {'tenant_id': '',  # tenant intentionally not set
@@ -603,8 +637,8 @@ class L3_NAT_db_mixin(l3.RouterPluginBase):
                      'name': ''}})
                 # Ensure IP addresses are allocated on external port
                 if not external_port['fixed_ips']:
-                    msg = "Unable to find any IP address on external network"
-                    # remove the external port
+                    msg = _("Unable to find any IP address on external "
+                            "network")
                     raise q_exc.BadRequest(resource='floatingip', msg=msg)
 
                 floating_fixed_ip = external_port['fixed_ips'][0]
@@ -624,14 +658,17 @@ class L3_NAT_db_mixin(l3.RouterPluginBase):
         # TODO(salvatore-orlando): Avoid broad catch
         # Maybe by introducing base class for L3 exceptions
         except q_exc.BadRequest:
-            LOG.exception("Unable to create Floating ip due to a "
-                          "malformed request")
+            LOG.exception(_("Unable to create Floating ip due to a "
+                            "malformed request"))
             raise
         except Exception:
-            LOG.exception("Floating IP association failed")
-            # Remove the port created for internal purposes
+            LOG.exception(_("Floating IP association failed"))
             raise
-
+        router_id = floatingip_db['router_id']
+        if router_id:
+            routers = self.get_sync_data(context.elevated(), [router_id])
+            l3_rpc_agent_api.L3AgentNotify.routers_updated(context, routers,
+                                                           'create_floatingip')
         return self._make_floatingip_dict(floatingip_db)
 
     def update_floatingip(self, context, id, floatingip):
@@ -641,27 +678,51 @@ class L3_NAT_db_mixin(l3.RouterPluginBase):
             fip['tenant_id'] = floatingip_db['tenant_id']
             fip['id'] = id
             fip_port_id = floatingip_db['floating_port_id']
+            before_router_id = floatingip_db['router_id']
             self._update_fip_assoc(context, fip, floatingip_db,
                                    self.get_port(context.elevated(),
                                                  fip_port_id))
+        router_ids = []
+        if before_router_id:
+            router_ids.append(before_router_id)
+        router_id = floatingip_db['router_id']
+        if router_id and router_id != before_router_id:
+            router_ids.append(router_id)
+        if router_ids:
+            routers = self.get_sync_data(context.elevated(), router_ids)
+            l3_rpc_agent_api.L3AgentNotify.routers_updated(context, routers,
+                                                           'update_floatingip')
         return self._make_floatingip_dict(floatingip_db)
 
     def delete_floatingip(self, context, id):
         floatingip = self._get_floatingip(context, id)
+        router_id = floatingip['router_id']
         with context.session.begin(subtransactions=True):
             context.session.delete(floatingip)
             self.delete_port(context.elevated(),
                              floatingip['floating_port_id'],
                              l3_port_check=False)
+        if router_id:
+            routers = self.get_sync_data(context.elevated(), [router_id])
+            l3_rpc_agent_api.L3AgentNotify.routers_updated(context, routers,
+                                                           'delete_floatingip')
 
     def get_floatingip(self, context, id, fields=None):
         floatingip = self._get_floatingip(context, id)
         return self._make_floatingip_dict(floatingip, fields)
 
-    def get_floatingips(self, context, filters=None, fields=None):
+    def get_floatingips(self, context, filters=None, fields=None,
+                        sorts=None, limit=None, marker=None,
+                        page_reverse=False):
+        marker_obj = self._get_marker_obj(context, 'floatingip', limit,
+                                          marker)
         return self._get_collection(context, FloatingIP,
                                     self._make_floatingip_dict,
-                                    filters=filters, fields=fields)
+                                    filters=filters, fields=fields,
+                                    sorts=sorts,
+                                    limit=limit,
+                                    marker_obj=marker_obj,
+                                    page_reverse=page_reverse)
 
     def get_floatingips_count(self, context, filters=None):
         return self._get_collection_count(context, FloatingIP,
@@ -696,6 +757,7 @@ class L3_NAT_db_mixin(l3.RouterPluginBase):
             try:
                 fip_qry = context.session.query(FloatingIP)
                 floating_ip = fip_qry.filter_by(fixed_port_id=port_id).one()
+                router_id = floating_ip['router_id']
                 floating_ip.update({'fixed_port_id': None,
                                     'fixed_ip_address': None,
                                     'router_id': None})
@@ -703,8 +765,11 @@ class L3_NAT_db_mixin(l3.RouterPluginBase):
                 return
             except exc.MultipleResultsFound:
                 # should never happen
-                raise Exception('Multiple floating IPs found for port %s'
+                raise Exception(_('Multiple floating IPs found for port %s')
                                 % port_id)
+        if router_id:
+            routers = self.get_sync_data(context.elevated(), [router_id])
+            l3_rpc_agent_api.L3AgentNotify.routers_updated(context, routers)
 
     def _check_l3_view_auth(self, context, network):
         return policy.check(context,
@@ -770,7 +835,7 @@ class L3_NAT_db_mixin(l3.RouterPluginBase):
                 network_id=net_id).delete()
 
     def _filter_nets_l3(self, context, nets, filters):
-        vals = filters.get('router:external', [])
+        vals = filters and filters.get('router:external', [])
         if not vals:
             return nets
 
@@ -780,3 +845,150 @@ class L3_NAT_db_mixin(l3.RouterPluginBase):
             return [n for n in nets if n['id'] in ext_nets]
         else:
             return [n for n in nets if n['id'] not in ext_nets]
+
+    def _get_sync_routers(self, context, router_ids=None, active=None):
+        """Query routers and their gw ports for l3 agent.
+
+        Query routers with the router_ids. The gateway ports, if any,
+        will be queried too.
+        l3 agent has an option to deal with only one router id. In addition,
+        when we need to notify the agent the data about only one router
+        (when modification of router, its interfaces, gw_port and floatingips),
+        we will have router_ids.
+        @param router_ids: the list of router ids which we want to query.
+                           if it is None, all of routers will be queried.
+        @return: a list of dicted routers with dicted gw_port populated if any
+        """
+        router_query = context.session.query(Router)
+        if router_ids:
+            if 1 == len(router_ids):
+                router_query = router_query.filter(Router.id == router_ids[0])
+            else:
+                router_query = router_query.filter(Router.id.in_(router_ids))
+        if active is not None:
+            router_query = router_query.filter(Router.admin_state_up == active)
+        routers = router_query.all()
+        gw_port_ids = []
+        if not routers:
+            return []
+        for router in routers:
+            gw_port_id = router.gw_port_id
+            if gw_port_id:
+                gw_port_ids.append(gw_port_id)
+        gw_ports = []
+        if gw_port_ids:
+            gw_ports = self.get_sync_gw_ports(context, gw_port_ids)
+        gw_port_id_gw_port_dict = {}
+        for gw_port in gw_ports:
+            gw_port_id_gw_port_dict[gw_port['id']] = gw_port
+        router_id_gw_port_id_dict = {}
+        for router in routers:
+            router_id_gw_port_id_dict[router.id] = router.gw_port_id
+        routers_list = [self._make_router_dict(c, None) for c in routers]
+        for router in routers_list:
+            gw_port_id = router_id_gw_port_id_dict[router['id']]
+            if gw_port_id:
+                router['gw_port'] = gw_port_id_gw_port_dict[gw_port_id]
+        return routers_list
+
+    def _get_sync_floating_ips(self, context, router_ids):
+        """Query floating_ips that relate to list of router_ids."""
+        if not router_ids:
+            return []
+        return self.get_floatingips(context, {'router_id': router_ids})
+
+    def get_sync_gw_ports(self, context, gw_port_ids):
+        if not gw_port_ids:
+            return []
+        filters = {'id': gw_port_ids}
+        gw_ports = self.get_ports(context, filters)
+        if gw_ports:
+            self._populate_subnet_for_ports(context, gw_ports)
+        return gw_ports
+
+    def get_sync_interfaces(self, context, router_ids,
+                            device_owner=DEVICE_OWNER_ROUTER_INTF):
+        """Query router interfaces that relate to list of router_ids."""
+        if not router_ids:
+            return []
+        filters = {'device_id': router_ids,
+                   'device_owner': [device_owner]}
+        interfaces = self.get_ports(context, filters)
+        if interfaces:
+            self._populate_subnet_for_ports(context, interfaces)
+        return interfaces
+
+    def _populate_subnet_for_ports(self, context, ports):
+        """Populate ports with subnet.
+
+        These ports already have fixed_ips populated.
+        """
+        if not ports:
+            return
+        subnet_id_ports_dict = {}
+        for port in ports:
+            fixed_ips = port.get('fixed_ips', [])
+            if len(fixed_ips) > 1:
+                LOG.info(_("Ignoring multiple IPs on router port %s"),
+                         port['id'])
+                continue
+            elif not fixed_ips:
+                # Skip ports without IPs, which can occur if a subnet
+                # attached to a router is deleted
+                LOG.info(_("Skipping port %s as no IP is configure on it"),
+                         port['id'])
+                continue
+            fixed_ip = fixed_ips[0]
+            my_ports = subnet_id_ports_dict.get(fixed_ip['subnet_id'], [])
+            my_ports.append(port)
+            subnet_id_ports_dict[fixed_ip['subnet_id']] = my_ports
+        if not subnet_id_ports_dict:
+            return
+        filters = {'id': subnet_id_ports_dict.keys()}
+        fields = ['id', 'cidr', 'gateway_ip']
+        subnet_dicts = self.get_subnets(context, filters, fields)
+        for subnet_dict in subnet_dicts:
+            ports = subnet_id_ports_dict.get(subnet_dict['id'], [])
+            for port in ports:
+                # TODO(gongysh) stash the subnet into fixed_ips
+                # to make the payload smaller.
+                port['subnet'] = {'id': subnet_dict['id'],
+                                  'cidr': subnet_dict['cidr'],
+                                  'gateway_ip': subnet_dict['gateway_ip']}
+
+    def _process_sync_data(self, routers, interfaces, floating_ips):
+        routers_dict = {}
+        for router in routers:
+            routers_dict[router['id']] = router
+        for floating_ip in floating_ips:
+            router = routers_dict.get(floating_ip['router_id'])
+            if router:
+                router_floatingips = router.get(l3_constants.FLOATINGIP_KEY,
+                                                [])
+                router_floatingips.append(floating_ip)
+                router[l3_constants.FLOATINGIP_KEY] = router_floatingips
+        for interface in interfaces:
+            router = routers_dict.get(interface['device_id'])
+            if router:
+                router_interfaces = router.get(l3_constants.INTERFACE_KEY, [])
+                router_interfaces.append(interface)
+                router[l3_constants.INTERFACE_KEY] = router_interfaces
+        return routers_dict.values()
+
+    def get_sync_data(self, context, router_ids=None, active=None):
+        """Query routers and their related floating_ips, interfaces."""
+        with context.session.begin(subtransactions=True):
+            routers = self._get_sync_routers(context,
+                                             router_ids=router_ids,
+                                             active=active)
+            router_ids = [router['id'] for router in routers]
+            floating_ips = self._get_sync_floating_ips(context, router_ids)
+            interfaces = self.get_sync_interfaces(context, router_ids)
+        return self._process_sync_data(routers, interfaces, floating_ips)
+
+    def get_external_network_id(self, context):
+        nets = self.get_networks(context, {'router:external': [True]})
+        if len(nets) > 1:
+            raise q_exc.TooManyExternalNetworks()
+        else:
+            return nets[0]['id'] if nets else None
