@@ -1,6 +1,6 @@
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
 
-#    Copyright 2011 OpenStack LLC
+#    Copyright 2011 OpenStack Foundation
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
 #    not use this file except in compliance with the License. You may obtain
@@ -15,16 +15,16 @@
 #    under the License.
 
 import os
-import subprocess
 
 import mock
-import unittest2 as unittest
 
+from quantum.common import utils
 from quantum.rootwrap import filters
 from quantum.rootwrap import wrapper
+from quantum.tests import base
 
 
-class RootwrapTestCase(unittest.TestCase):
+class RootwrapTestCase(base.BaseTestCase):
 
     def setUp(self):
         super(RootwrapTestCase, self).setUp()
@@ -80,7 +80,7 @@ class RootwrapTestCase(unittest.TestCase):
         self.assertEqual(env.get('QUANTUM_NETWORK_ID'), 'foobar')
 
     def test_KillFilter(self):
-        p = subprocess.Popen(["/bin/sleep", "5"])
+        p = utils.subprocess_popen(["/bin/sleep", "5"])
         f = filters.KillFilter("root", "/bin/sleep", "-9", "-HUP")
         f2 = filters.KillFilter("root", "/usr/bin/sleep", "-9", "-HUP")
         usercmd = ['kill', '-ALRM', p.pid]
